@@ -119,8 +119,7 @@ class Element(ElementBase):
         method is greedy
         """
         self.logger.debug(f"{inspect.currentframe().f_code.co_name}")
-
-        self._get_driver()
+        start_time = time.time()
 
         # [Step] Normalize locator
         step = "Normalizing locator"
@@ -146,50 +145,59 @@ class Element(ElementBase):
         self.logger.debug(f"[{step}] started")
 
         self.logger.info(f"{locator=}")
-        try:
-            wait = WebDriverWait(driver=self.driver, timeout=timeout,
-                                 poll_frequency=poll_frequency, ignored_exceptions=ignored_exceptions)
-            wait.until(EC.presence_of_element_located(locator))
-            native_elements = self.driver.find_elements(*locator)
+        while time.time() - start_time < self.timeout:
+            try:
+                self._get_driver()
+                wait = WebDriverWait(
+                    driver=self.driver,
+                    timeout=timeout,
+                    poll_frequency=poll_frequency,
+                    ignored_exceptions=ignored_exceptions,
+                )
+                wait.until(EC.presence_of_element_located(locator))
+                native_elements = self.driver.find_elements(*locator)
 
-            elements = []
-            for native_element in native_elements:
-                el_class = native_element.get_attribute('class')
-                el_text = native_element.get_attribute('text')
-                el_content_desc = native_element.get_attribute('content-desc')
-                el_checkable = native_element.get_attribute('checkable')
-                el_checked = native_element.get_attribute('checked')
-                el_clickable = native_element.get_attribute('clickable')
-                el_enabled = native_element.get_attribute('enabled')
-                el_focusable = native_element.get_attribute('focusable')
-                el_focused = native_element.get_attribute('focused')
-                el_long_clickable = native_element.get_attribute('long-clickable')
-                el_scrollable = native_element.get_attribute('scrollable')
-                el_selected = native_element.get_attribute('selected')
-                el_displayed = native_element.get_attribute('displayed')
-                element = Element(locator={'class': el_class,
-                                           'text': el_text,
-                                           'content-desc': el_content_desc,
-                                           'checkable': el_checkable,
-                                           'checked': el_checked,
-                                           'clickable': el_clickable,
-                                           'enabled': el_enabled,
-                                           'focusable': el_focusable,
-                                           'focused': el_focused,
-                                           'long-clickable': el_long_clickable,
-                                           'scrollable': el_scrollable,
-                                           'selected': el_selected,
-                                           'displayed': el_displayed},
-                                  base=self.base,
-                                  timeout=timeout,
-                                  poll_frequency=poll_frequency,
-                                  ignored_exceptions=ignored_exceptions,
-                                  contains=contains)
-                elements.append(element)
-            return elements
-        except WebDriverException:
-            raise   # вот тут нужно вернуть адекватную ошибку
+                elements = []
+                for native_element in native_elements:
+                    # [Extract attributes]
+                    attributes = {
+                        attr: native_element.get_attribute(attr) for attr in [
+                            'class', 'text', 'content-desc', 'checkable', 'checked',
+                            'clickable', 'enabled', 'focusable', 'focused',
+                            'long-clickable', 'scrollable', 'selected', 'displayed'
+                        ]
+                    }
+                    element = Element(
+                        locator=attributes,
+                        base=self.base,
+                        timeout=timeout,
+                        poll_frequency=poll_frequency,
+                        ignored_exceptions=ignored_exceptions,
+                        contains=contains
+                    )
+                    elements.append(element)
+                return elements
 
+            except NoSuchDriverException as error:
+                self._handle_driver_error(error)
+
+            except InvalidSessionIdException as error:
+                self._handle_driver_error(error)
+
+            except StaleElementReferenceException as error:
+                self.logger.warning(f"Stale element encountered, retrying... | {error}")
+                continue
+
+            except TimeoutException as error:
+                self.logger.warning(f"Timeout while waiting for presence of element | {error}")
+                continue
+
+        # [Fail Path] – Escaped while-loop without result
+        msg = f"No elements found by locator: {locator} within {timeout} seconds"
+        screen = self.driver.get_screenshot_as_base64() if self.driver else None
+        stacktrace = traceback.format_exc()
+
+        raise GeneralElementException(msg=msg, screen=screen, stacktrace=stacktrace)
 
     def get_attributes(self) -> Optional[Dict[str, str]]:
         """Fetch all XML attributes of the element by matching locator against page source.
@@ -1053,7 +1061,7 @@ class Element(ElementBase):
                 self.scroll_down(percent=percent, speed=speed, return_bool=True)
             except (
                     NoSuchDriverException, InvalidSessionIdException, AttributeError
-                    ) as error:
+            ) as error:
                 self._handle_driver_error(error)
             except StaleElementReferenceException as error:
                 self.logger.error(f"{error}\ncontinue")
@@ -2043,23 +2051,38 @@ class Element(ElementBase):
             bool: True if the element is found, False otherwise.
         """
         self.logger.debug(f"{inspect.currentframe().f_code.co_name}")
-        try:
-            resolved_locator = self.handle_locator(self.locator, self.contains)
-            if not resolved_locator:
-                self.logger.error("Resolved locator is None or invalid")
+        start_time = time.time()
+        while time.time() - start_time < self.timeout:
+            try:
+                resolved_locator = self.handle_locator(self.locator, self.contains)
+                if not resolved_locator:
+                    self.logger.error("Resolved locator is None or invalid")
+                    if return_bool:
+                        return False
+                    return cast('Element', self)
+                WebDriverWait(self.base.driver, timeout, poll_frequency).until(
+                    conditions.present(resolved_locator)
+                )
+                if return_bool:
+                    return True
+                return cast('Element', self)
+            except TimeoutException:
                 if return_bool:
                     return False
                 return cast('Element', self)
-            WebDriverWait(self.base.driver, timeout, poll_frequency).until(
-                conditions.present(resolved_locator)
-            )
-            if return_bool:
-                return True
-            return cast('Element', self)
-        except TimeoutException:
-            if return_bool:
-                return False
-            return cast('Element', self)
+            except NoSuchDriverException as error:
+                self._handle_driver_error(error)
+            except InvalidSessionIdException as error:
+                self._handle_driver_error(error)
+            except StaleElementReferenceException as error:
+                self.logger.error(f"{error}")
+                continue
+            except WebDriverException as error:
+                self._handle_driver_error(error)
+            except Exception as error:
+                self.logger.error(f"{error}")
+                continue
+        return False
 
     def wait_visible(self, timeout: int = 10, poll_frequency: float = 0.5, return_bool: bool = False) -> Union[
         bool, 'Element']:
@@ -2074,24 +2097,39 @@ class Element(ElementBase):
             bool: True if the element becomes visible, False otherwise.
         """
         self.logger.debug(f"{inspect.currentframe().f_code.co_name}")
-        try:
-            resolved_locator = self.handle_locator(self.locator, self.contains)
-            if not resolved_locator:
-                self.logger.error("Resolved locator is None or invalid")
+        start_time = time.time()
+        while time.time() - start_time < self.timeout:
+            try:
+                resolved_locator = self.handle_locator(self.locator, self.contains)
+                if not resolved_locator:
+                    self.logger.error("Resolved locator is None or invalid")
+                    if return_bool:
+                        return False
+                    return cast('Element', self)
+
+                WebDriverWait(self.base.driver, timeout, poll_frequency).until(
+                    conditions.visible(resolved_locator)
+                )
+                if return_bool:
+                    return True
+                return cast('Element', self)
+            except TimeoutException:
                 if return_bool:
                     return False
                 return cast('Element', self)
-
-            WebDriverWait(self.base.driver, timeout, poll_frequency).until(
-                conditions.visible(resolved_locator)
-            )
-            if return_bool:
-                return True
-            return cast('Element', self)
-        except TimeoutException:
-            if return_bool:
-                return False
-            return cast('Element', self)
+            except NoSuchDriverException as error:
+                self._handle_driver_error(error)
+            except InvalidSessionIdException as error:
+                self._handle_driver_error(error)
+            except StaleElementReferenceException as error:
+                self.logger.error(f"{error}")
+                continue
+            except WebDriverException as error:
+                self._handle_driver_error(error)
+            except Exception as error:
+                self.logger.error(f"{error}")
+                continue
+        return False
 
     def wait_clickable(self, timeout: int = 10, poll_frequency: float = 0.5, return_bool: bool = False) -> Union[
         bool, 'Element']:
@@ -2106,24 +2144,39 @@ class Element(ElementBase):
             bool: True if the element becomes clickable, False otherwise.
         """
         self.logger.debug(f"{inspect.currentframe().f_code.co_name}")
-        try:
-            resolved_locator = self.handle_locator(self.locator, self.contains)
-            if not resolved_locator:
-                self.logger.error("Resolved locator is None or invalid")
+        start_time = time.time()
+        while time.time() - start_time < self.timeout:
+            try:
+                resolved_locator = self.handle_locator(self.locator, self.contains)
+                if not resolved_locator:
+                    self.logger.error("Resolved locator is None or invalid")
+                    if return_bool:
+                        return False
+                    return cast('Element', self)
+
+                WebDriverWait(self.base.driver, timeout, poll_frequency).until(
+                    conditions.clickable(resolved_locator)
+                )
+                if return_bool:
+                    return True
+                return cast('Element', self)
+            except TimeoutException:
                 if return_bool:
                     return False
                 return cast('Element', self)
-
-            WebDriverWait(self.base.driver, timeout, poll_frequency).until(
-                conditions.clickable(resolved_locator)
-            )
-            if return_bool:
-                return True
-            return cast('Element', self)
-        except TimeoutException:
-            if return_bool:
-                return False
-            return cast('Element', self)
+            except NoSuchDriverException as error:
+                self._handle_driver_error(error)
+            except InvalidSessionIdException as error:
+                self._handle_driver_error(error)
+            except StaleElementReferenceException as error:
+                self.logger.error(f"{error}")
+                continue
+            except WebDriverException as error:
+                self._handle_driver_error(error)
+            except Exception as error:
+                self.logger.error(f"{error}")
+                continue
+        return False
 
     def wait_for_not(self, timeout: int = 10, poll_frequency: float = 0.5, return_bool: bool = False) -> Union[
         bool, 'Element']:
@@ -2138,22 +2191,37 @@ class Element(ElementBase):
             bool: True if the element disappears, False otherwise.
         """
         self.logger.debug(f"{inspect.currentframe().f_code.co_name}")
-        try:
-            resolved_locator = self.handle_locator(self.locator, self.contains)
-            if not resolved_locator:
+        start_time = time.time()
+        while time.time() - start_time < self.timeout:
+            try:
+                resolved_locator = self.handle_locator(self.locator, self.contains)
+                if not resolved_locator:
+                    if return_bool:
+                        return False
+                    return cast('Element', self)
+                WebDriverWait(self.base.driver, timeout, poll_frequency).until(
+                    conditions.not_present(resolved_locator)
+                )
+                if return_bool:
+                    return True
+                return cast('Element', self)
+            except TimeoutException:
                 if return_bool:
                     return False
                 return cast('Element', self)
-            WebDriverWait(self.base.driver, timeout, poll_frequency).until(
-                conditions.not_present(resolved_locator)
-            )
-            if return_bool:
-                return True
-            return cast('Element', self)
-        except TimeoutException:
-            if return_bool:
-                return False
-            return cast('Element', self)
+            except NoSuchDriverException as error:
+                self._handle_driver_error(error)
+            except InvalidSessionIdException as error:
+                self._handle_driver_error(error)
+            except StaleElementReferenceException as error:
+                self.logger.error(f"{error}")
+                continue
+            except WebDriverException as error:
+                self._handle_driver_error(error)
+            except Exception as error:
+                self.logger.error(f"{error}")
+                continue
+        return False
 
     def wait_for_not_visible(self, timeout: int = 10, poll_frequency: float = 0.5, return_bool: bool = False) -> Union[
         bool, 'Element']:
@@ -2168,25 +2236,40 @@ class Element(ElementBase):
             bool: True if the element becomes invisible, False otherwise.
         """
         self.logger.debug(f"{inspect.currentframe().f_code.co_name}")
-        try:
-            resolved_locator = self.handle_locator(self.locator, self.contains)
-            if not resolved_locator:
+        start_time = time.time()
+        while time.time() - start_time < self.timeout:
+            try:
+                resolved_locator = self.handle_locator(self.locator, self.contains)
+                if not resolved_locator:
+                    if return_bool:
+                        return False
+                    return cast('Element', self)
+                WebDriverWait(self.base.driver, timeout, poll_frequency).until(
+                    conditions.not_visible(resolved_locator)
+                )
+                if return_bool:
+                    return True
+                return cast('Element', self)
+            except TimeoutException:
                 if return_bool:
                     return False
                 return cast('Element', self)
-            WebDriverWait(self.base.driver, timeout, poll_frequency).until(
-                conditions.not_visible(resolved_locator)
-            )
-            if return_bool:
-                return True
-            return cast('Element', self)
-        except TimeoutException:
-            if return_bool:
-                return False
-            return cast('Element', self)
+            except NoSuchDriverException as error:
+                self._handle_driver_error(error)
+            except InvalidSessionIdException as error:
+                self._handle_driver_error(error)
+            except StaleElementReferenceException as error:
+                self.logger.error(f"{error}")
+                continue
+            except WebDriverException as error:
+                self._handle_driver_error(error)
+            except Exception as error:
+                self.logger.error(f"{error}")
+                continue
+        return False
 
     def wait_for_not_clickable(self, timeout: int = 10, poll_frequency: float = 0.5, return_bool: bool = False) -> \
-    Union[bool, 'Element']:
+            Union[bool, 'Element']:
         """Waits until the element becomes not clickable.
 
         Args:
@@ -2198,23 +2281,38 @@ class Element(ElementBase):
             bool: True if the element becomes not clickable, False otherwise.
         """
         self.logger.debug(f"{inspect.currentframe().f_code.co_name}")
-        try:
-            resolved_locator = self.handle_locator(self.locator, self.contains)
-            if not resolved_locator:
-                self.logger.error("Resolved locator is None or invalid")
+        start_time = time.time()
+        while time.time() - start_time < self.timeout:
+            try:
+                resolved_locator = self.handle_locator(self.locator, self.contains)
+                if not resolved_locator:
+                    self.logger.error("Resolved locator is None or invalid")
+                    if return_bool:
+                        return False
+                    return cast('Element', self)
+                WebDriverWait(self.base.driver, timeout, poll_frequency).until(
+                    conditions.not_clickable(resolved_locator)
+                )
+                if return_bool:
+                    return True
+                return cast('Element', self)
+            except TimeoutException:
                 if return_bool:
                     return False
                 return cast('Element', self)
-            WebDriverWait(self.base.driver, timeout, poll_frequency).until(
-                conditions.not_clickable(resolved_locator)
-            )
-            if return_bool:
-                return True
-            return cast('Element', self)
-        except TimeoutException:
-            if return_bool:
-                return False
-            return cast('Element', self)
+            except NoSuchDriverException as error:
+                self._handle_driver_error(error)
+            except InvalidSessionIdException as error:
+                self._handle_driver_error(error)
+            except StaleElementReferenceException as error:
+                self.logger.error(f"{error}")
+                continue
+            except WebDriverException as error:
+                self._handle_driver_error(error)
+            except Exception as error:
+                self.logger.error(f"{error}")
+                continue
+        return False
 
     @property
     def should(self) -> 'Should':
@@ -2237,7 +2335,6 @@ class Element(ElementBase):
             contains=self.contains
         )
 
-
     def _contains_to_xpath(self, xpath: Tuple[str, str]) -> Tuple[str, str]:
         """
         Applies contains(...) only to specific attributes in XPath expression.
@@ -2263,9 +2360,4 @@ class Element(ElementBase):
 
         for attr, pattern in patterns.items():
             value = re.sub(pattern, lambda m: f"contains(@{attr}, '{m.group(1)}')", value)
-
         return strategy, value
-
-
-
-
